"""场景：美化报表 — 让 Agent 添加格式。"""

from .conftest import invoke


class TestFormatReport:

    def test_bold_header_and_freeze(self, agent, xlsx_env, renderer):
        """Agent 能给表头加粗并冻结首行。"""
        result, wb = invoke(
            agent,
            "把表头行加粗，字号设为14，并冻结首行",
            xlsx_env,
            renderer,
        )
        ws = wb["Sheet1"]
        assert ws["A1"].font.bold is True
        assert ws.freeze_panes is not None

    def test_fill_background(self, agent, xlsx_env, renderer):
        """Agent 能给表头添加背景色。"""
        result, wb = invoke(
            agent,
            "给表头行（第一行）添加蓝色背景",
            xlsx_env,
            renderer,
        )
        ws = wb["Sheet1"]
        assert ws["A1"].fill.fill_type == "solid"



"""Excel 文件读取工具。"""

from __future__ import annotations

import logging
from pathlib import Path
from typing import Annotated, Optional

from langchain_core.tools import ToolException, tool
from langgraph.prebuilt import InjectedState

from ._helpers import get_input_file

logger = logging.getLogger(__name__)


def _read_excel_impl(
    file_path: str, sheet_name: str | None = None, max_rows: int = 50
) -> str:
    import pandas as pd

    path = Path(file_path)
    if not path.exists():
        logger.error("文件不存在: %s", file_path)
        raise FileNotFoundError(f"文件不存在: {file_path}")

    try:
        if sheet_name:
            df = pd.read_excel(file_path, sheet_name=sheet_name)
            preview = df.head(max_rows).to_string()
            return (
                f"Sheet: {sheet_name}\n"
                f"Shape: {df.shape}\n"
                f"Columns: {list(df.columns)}\n\n{preview}"
            )

        all_sheets = pd.read_excel(file_path, sheet_name=None)
        parts = []
        for name, df in all_sheets.items():
            preview = df.head(max_rows).to_string()
            parts.append(
                f"=== Sheet: {name} | Shape: {df.shape} "
                f"| Columns: {list(df.columns)} ===\n{preview}"
            )
        return "\n\n".join(parts)
    except FileNotFoundError:
        raise
    except Exception as e:
        logger.error("读取 Excel 失败: %s", e)
        raise ToolException(f"读取 Excel 失败: {e}") from e


@tool
def read_input(
    state: Annotated[dict, InjectedState],
    sheet_name: Optional[str] = None,
    max_rows: int = 50,
) -> str:
    """读取输入 Excel 文件内容，返回数据预览。

    Args:
        sheet_name: 指定工作表名称，不指定则读取所有工作表
        max_rows: 最大返回行数，默认 50
    """
    file_path = get_input_file(state)
    result = _read_excel_impl(file_path, sheet_name, max_rows)
    return f"[输入文件路径: {file_path}]\n{result}"


@tool
def read_excel(
    file_path: str,
    sheet_name: Optional[str] = None,
    max_rows: int = 50,
) -> str:
    """读取任意 Excel 文件内容，返回数据预览。

    Args:
        file_path: Excel 文件路径
        sheet_name: 指定工作表名称，不指定则读取所有工作表
        max_rows: 最大返回行数，默认 50
    """
    return _read_excel_impl(file_path, sheet_name, max_rows)



"""LangGraph Studio 入口 — 自动发现所有 Agent。

启动: langgraph dev
需要 .env 文件配置: API_KEY, API_BASE_URL, MODEL_NAME

新增 Agent 子包后，只需在 langgraph.json 中添加一行引用即可。
变量命名规则: {agent.name}_graph （如 xlsx_graph）
"""

import importlib
import os
import pkgutil

from dotenv import load_dotenv
from langchain_openai import ChatOpenAI

import ai_hub_agents
from ai_hub_agents.core import BaseAgent

load_dotenv()

llm = ChatOpenAI(
    api_key=os.environ["API_KEY"],
    base_url=os.environ["API_BASE_URL"],
    model=os.environ["MODEL_NAME"],
    temperature=0,
)

for _, module_name, is_pkg in pkgutil.iter_modules(ai_hub_agents.__path__):
    if is_pkg and module_name != "core":
        importlib.import_module(f"ai_hub_agents.{module_name}")

for _cls in BaseAgent.__subclasses__():
    _agent = _cls.create(llm)
    globals()[f"{_cls.name}_graph"] = _agent._graph



"""测试环境工具。

提供快速加载 .env 环境变量并创建 LLM 实例的公共函数，
避免每个测试脚本重复编写样板代码。
"""

from __future__ import annotations

import logging
import os
import signal
import sys
from typing import Any

from dotenv import load_dotenv
from langchain_core.language_models import BaseChatModel
from langchain_openai import ChatOpenAI

_REQUIRED_KEYS = ("API_KEY", "API_BASE_URL", "MODEL_NAME")


def load_test_llm(**kwargs: Any) -> BaseChatModel:
    """从 .env 加载环境变量并创建 ChatOpenAI 实例。

    自动调用 load_dotenv()，检查必需的环境变量，
    返回可直接使用的 LLM 实例。kwargs 透传给 ChatOpenAI。
    """
    load_dotenv()
    missing = [k for k in _REQUIRED_KEYS if k not in os.environ]
    if missing:
        raise ValueError(f"缺少环境变量: {', '.join(missing)}，请在 .env 中配置")

    defaults: dict[str, Any] = dict(
        api_key=os.environ["API_KEY"],
        base_url=os.environ["API_BASE_URL"],
        model=os.environ["MODEL_NAME"],
        temperature=0,
        streaming=True,
    )
    defaults.update(kwargs)
    return ChatOpenAI(**defaults)


def setup_logging(level: int = logging.INFO) -> None:
    """配置日志，默认 INFO 级别以显示渲染器输出，同时抑制第三方库噪音。"""
    signal.signal(signal.SIGINT, signal.SIG_DFL)
    logging.basicConfig(
        level=level,
        format="%(asctime)s %(message)s",
        datefmt="%H:%M:%S",
        stream=sys.stdout,
        force=True,
    )
    for name in ("httpx", "openai", "httpcore"):
        logging.getLogger(name).setLevel(logging.WARNING)



"""xlsx 工具集 — 统一导出。"""

from .cell_ops import clear_range, read_cells, write_cells, write_range
from .chart_ops import create_chart
from .data_ops import (
    add_conditional_format,
    add_data_validation,
    copy_range,
    find_replace,
    sort_range,
)
from .excel_read import read_excel, read_input
from .execute_python import execute_python
from .formatting import (
    auto_fit_columns,
    format_cells,
    freeze_panes,
    merge_cells,
    set_column_width,
    set_row_height,
    unmerge_cells,
)
from .office_xml import pack_office, unpack_office, validate_office
from .recalc import recalc_formulas
from .row_col_ops import delete_cols, delete_rows, insert_cols, insert_rows
from .sheet_ops import add_sheet, copy_sheet, list_sheets, remove_sheet, rename_sheet

ALL_TOOLS = (
    # 读取
    read_input,
    read_excel,
    read_cells,
    # 单元格写入
    write_cells,
    write_range,
    clear_range,
    # 工作表管理
    list_sheets,
    add_sheet,
    remove_sheet,
    rename_sheet,
    copy_sheet,
    # 行列操作
    insert_rows,
    delete_rows,
    insert_cols,
    delete_cols,
    # 格式化
    format_cells,
    set_column_width,
    set_row_height,
    merge_cells,
    unmerge_cells,
    freeze_panes,
    auto_fit_columns,
    # 数据操作
    find_replace,
    copy_range,
    sort_range,
    add_conditional_format,
    add_data_validation,
    # 图表
    create_chart,
    # 公式重算
    recalc_formulas,
    # XML 操作
    unpack_office,
    pack_office,
    validate_office,
    # 兜底
    execute_python,
)

for _t in ALL_TOOLS:
    _t.handle_tool_error = True

__all__ = [t.name for t in ALL_TOOLS] + ["ALL_TOOLS"]



"""xlsx Agent 集成测试公共 fixture。"""

from __future__ import annotations

import pytest
from openpyxl import Workbook, load_workbook

from ai_hub_agents.test import ColorStreamRenderer, load_test_llm, setup_logging
from ai_hub_agents.xlsx import XlsxAgent


def pytest_configure(config):
    setup_logging()


@pytest.fixture(scope="session")
def agent():
    """整个测试会话复用同一个 agent 实例。"""
    llm = load_test_llm()
    return XlsxAgent.create(llm)


@pytest.fixture()
def xlsx_env(tmp_path):
    """创建带有示例数据的 xlsx 文件，返回 (input_file, output_file) 路径。"""
    input_file = tmp_path / "input.xlsx"
    output_file = tmp_path / "output.xlsx"

    wb = Workbook()
    ws = wb.active
    ws.title = "Sheet1"
    ws.append(["商品名称", "品类", "下单次数", "退货人数", "销售额"])
    ws.append(["商品1", "口红", 182, 90, 13988])
    ws.append(["商品2", "数码", 807, 0, 33935])
    ws.append(["商品3", "图书", 810, 766, 472])
    ws.append(["商品4", "食品", 186, 144, 2100])
    ws.append(["商品5", "数码", 500, 0, 25000])
    ws.append(["商品6", "口红", 320, 187, 8900])
    ws.append(["商品7", "食品", 90, 0, 1200])
    wb.save(input_file)

    return str(input_file), str(output_file)


@pytest.fixture()
def renderer():
    return ColorStreamRenderer()


def invoke(agent, task: str, xlsx_env, renderer) -> tuple[str, load_workbook]:
    """公共调用函数：执行 agent 并返回 (结果文本, 输出workbook)。"""
    input_file, output_file = xlsx_env
    result = agent.invoke(
        task,
        callbacks=[renderer],
        input_file=input_file,
        output_file=output_file,
    )
    wb = load_workbook(output_file)
    return result, wb



"""场景：查看数据 — 让 Agent 读取并描述 Excel 内容。"""

from .conftest import invoke


class TestViewData:

    def test_describe_table_content(self, agent, xlsx_env, renderer):
        """Agent 能读取数据并描述表格内容。"""
        result, wb = invoke(agent, "读取输入文件，告诉我有哪些列和多少行数据", xlsx_env, renderer)
        assert "商品" in result or "品类" in result or "销售" in result



"""终端彩色流式渲染器。

作为 StreamCallback 的一种实现，将 Agent 的 stream 事件
以彩色 ANSI 格式通过 logging 输出。可作为模板编写其他渲染器
（如 Rich、JSON、WebSocket 等）。
"""

from __future__ import annotations

import logging
from typing import Any

from langchain_core.messages import AIMessage

from ai_hub_agents.core.callbacks import StreamCallback

logger = logging.getLogger("ai_hub_agents.stream")

CYAN = "\033[36m"
GREEN = "\033[32m"
YELLOW = "\033[33m"
RED = "\033[31m"
BOLD = "\033[1m"
DIM = "\033[2m"
RESET = "\033[0m"


class ColorStreamRenderer(StreamCallback):
    """终端彩色流式渲染器，通过 logging.info 输出。"""

    def on_stream_start(self) -> None:
        logger.info(f"{DIM}{'─' * 50}{RESET}")

    def on_stream_end(self, result: str) -> None:
        logger.info(f"{DIM}{'─' * 50}{RESET}")

    def on_ai_message(self, message: AIMessage) -> None:
        text = message.content[:200]
        logger.info(f"{CYAN}🤖 AI{RESET}  {text}")

    def on_tool_call(self, name: str, args: dict[str, Any]) -> None:
        logger.info(
            f"{YELLOW}🔧 Call{RESET}  "
            f"{BOLD}{name}{RESET} "
            f"{DIM}{args}{RESET}"
        )

    def on_tool_result(self, name: str, content: str) -> None:
        logger.info(
            f"{GREEN}✅ Result{RESET}  "
            f"{BOLD}{name}{RESET}: "
            f"{DIM}{content[:300]}{RESET}"
        )

    def on_tool_error(self, name: str, content: str) -> None:
        logger.info(
            f"{RED}❌ Error{RESET}  "
            f"{BOLD}{name}{RESET}: "
            f"{DIM}{content[:300]}{RESET}"
        )



"""场景：填写和修改数据 — 让 Agent 写入数据和公式。"""

from .conftest import invoke


class TestEditData:

    def test_add_summary_row(self, agent, xlsx_env, renderer):
        """Agent 能在最后一行添加汇总公式。"""
        result, wb = invoke(
            agent,
            "在数据最后一行的下方添加合计行，计算下单次数、退货人数、销售额的总和",
            xlsx_env,
            renderer,
        )
        ws = wb["Sheet1"]
        found_total = False
        for row in ws.iter_rows(min_row=2, max_col=5):
            if row[0].value and "合计" in str(row[0].value):
                found_total = True
                break
        assert found_total

    def test_find_and_replace(self, agent, xlsx_env, renderer):
        """Agent 能执行查找替换。"""
        result, wb = invoke(
            agent,
            "把所有的'口红'替换为'美妆'",
            xlsx_env,
            renderer,
        )
        ws = wb["Sheet1"]
        values = [ws.cell(row=r, column=2).value for r in range(2, 9)]
        assert "口红" not in values
        assert "美妆" in values



"""Python 代码执行工具（注入 State 路径）。"""

from __future__ import annotations

import logging
import os
import subprocess
import sys
from typing import Annotated

from langchain_core.tools import ToolException, tool
from langgraph.prebuilt import InjectedState

from ._helpers import get_input_file, get_output_file, with_file_lock

logger = logging.getLogger(__name__)


@tool
@with_file_lock
def execute_python(
    state: Annotated[dict, InjectedState],
    code: str,
) -> str:
    """执行 Python 代码进行 Excel 文件操作。

    可用库：openpyxl、pandas、pathlib。通过 print() 输出结果。

    代码中可通过环境变量获取文件路径：
    - os.environ['INPUT_FILE']  — 输入文件的绝对路径
    - os.environ['OUTPUT_FILE'] — 输出文件的绝对路径

    Args:
        code: 要执行的 Python 代码
    """
    env = os.environ.copy()
    env["INPUT_FILE"] = get_input_file(state)
    env["OUTPUT_FILE"] = get_output_file(state)

    try:
        result = subprocess.run(
            [sys.executable, "-c", code],
            capture_output=True,
            text=True,
            timeout=120,
            env=env,
        )
    except subprocess.TimeoutExpired as e:
        msg = "execute_python: 代码执行超时（120s）"
        logger.error(msg)
        raise ToolException(msg) from e

    output = result.stdout
    if result.stderr:
        output += f"\nStderr:\n{result.stderr}"
    if result.returncode != 0:
        logger.warning("execute_python: 代码执行失败 (exit %d)", result.returncode)
        output = f"[执行失败] Exit code {result.returncode}:\n{output}"
    return output or "(无输出)"



"""场景：创建图表 — 让 Agent 生成可视化图表。"""

from .conftest import invoke


class TestCreateChart:

    def test_create_revenue_chart(self, agent, xlsx_env, renderer):
        """Agent 能根据数据创建柱状图。"""
        result, wb = invoke(
            agent,
            "根据商品名称和销售额创建一个柱状图",
            xlsx_env,
            renderer,
        )
        ws = wb["Sheet1"]
        assert len(ws._charts) > 0



"""场景：公式重算 — Agent 写入公式后尝试重算应优雅处理。"""

from .conftest import invoke


class TestRecalc:

    def test_write_formula_and_recalc(self, agent, xlsx_env, renderer):
        """Agent 写入汇总公式后整体流程不报错。"""
        result, wb = invoke(
            agent,
            "在E9单元格写入销售额求和公式，然后重算公式",
            xlsx_env,
            renderer,
        )
        ws = wb["Sheet1"]
        found_formula = False
        for row in ws.iter_rows(min_row=2, max_col=5):
            for cell in row:
                if cell.value and str(cell.value).startswith("="):
                    found_formula = True
                    break
        assert found_formula



@app.post("/api/xlsx/process")
async def process(file_id: str, task: str):
    renderer = SSEStreamRenderer()

    async def run_agent():
        agent.invoke(task, callbacks=[renderer], input_file=..., output_file=...)

    async def event_stream():
        task_future = asyncio.get_event_loop().run_in_executor(None, run_agent_sync)
        while True:
            try:
                data = await asyncio.wait_for(renderer.queue.get(), timeout=0.5)
                yield f"data: {data}\n\n"
            except asyncio.TimeoutError:
                if task_future.done():
                    break
        # 发送下载链接
        yield f"data: {json.dumps({'type': 'done', 'download_url': f'/api/xlsx/download/{result_id}'})}\n\n"

    return StreamingResponse(event_stream(), media_type="text/event-stream")


import json
import asyncio

class SSEStreamRenderer(StreamCallback):
    """将 Agent 事件转为 SSE JSON，推入 asyncio.Queue。"""

    def __init__(self):
        self.queue: asyncio.Queue[str] = asyncio.Queue()

    def _send(self, data: dict):
        self.queue.put_nowait(json.dumps(data, ensure_ascii=False))

    def on_stream_start(self):
        self._send({"type": "stream_start"})

    def on_tool_call(self, name, args):
        self._send({
            "type": "tool_call",
            "name": name,
            "label": TOOL_LABELS.get(name, name),
            "args": _safe_summary(args),
        })

    def on_tool_result(self, name, content):
        self._send({
            "type": "tool_result",
            "name": name,
            "summary": content[:500],
            "success": True,
        })

    def on_tool_error(self, name, content):
        self._send({
            "type": "tool_error",
            "name": name,
            "summary": content[:500],
            "success": False,
        })

    def on_ai_message(self, message):
        self._send({
            "type": "ai_message",
            "content": message.content,
        })

    def on_stream_end(self, result):
        # file_id 由外部注入
        self._send({"type": "stream_end"})


"""输出文件自动初始化节点。"""

from __future__ import annotations

import shutil
from pathlib import Path


def auto_init_output(state: dict) -> dict:
    """前置节点：自动初始化输出文件。

    - 输出文件已存在 → 跳过（保留之前的修改）
    - 输入文件存在 → 复制到输出路径
    - 输入文件不存在 → 创建空工作簿
    """
    output = state["output_file"]
    Path(output).parent.mkdir(parents=True, exist_ok=True)
    input_file = state.get("input_file", "")
    if input_file and Path(input_file).exists():
        shutil.copy2(input_file, output)
    else:
        from openpyxl import Workbook

        Workbook().save(output)
    return {}



"""格式化工具。"""

from __future__ import annotations

from typing import Annotated, Optional

from langchain_core.tools import ToolException, tool
from langgraph.prebuilt import InjectedState
from openpyxl import load_workbook
from openpyxl.utils import range_boundaries

from ._helpers import ensure_dict, get_output_file, with_file_lock


def _load(state: dict):
    return load_workbook(get_output_file(state))


def _iter_cells(ws, range_str: str):
    """遍历范围内的所有单元格。"""
    min_col, min_row, max_col, max_row = range_boundaries(range_str)
    for row in ws.iter_rows(
        min_row=min_row, max_row=max_row,
        min_col=min_col, max_col=max_col,
    ):
        yield from row


@tool
@with_file_lock
def format_cells(
    state: Annotated[dict, InjectedState],
    sheet_name: str,
    range_str: str,
    font: Optional[dict | str] = None,
    fill: Optional[dict | str] = None,
    alignment: Optional[dict | str] = None,
    number_format: Optional[str] = None,
    border: Optional[dict | str] = None,
) -> str:
    """为输出文件中指定范围的单元格应用格式。所有格式参数均可选，按需传入。

    Args:
        sheet_name: 工作表名称
        range_str: 单元格范围，如 "A1:D10"
        font: 字体，如 {"name": "Arial", "size": 12, "bold": true, "italic": false, "color": "0000FF"}
        fill: 填充，如 {"color": "FFFF00", "fill_type": "solid"}
        alignment: 对齐，如 {"horizontal": "center", "vertical": "center", "wrap_text": true}
        number_format: 数字格式，如 "$#,##0", "0.0%", "#,##0;(#,##0)"
        border: 边框，如 {"style": "thin", "color": "000000"}。style 可为 thin/medium/thick/double/dashed/dotted
    """
    from openpyxl.styles import Alignment, Border, Font, PatternFill, Side

    font = ensure_dict(font)
    fill = ensure_dict(fill)
    alignment = ensure_dict(alignment)
    border = ensure_dict(border)

    output = get_output_file(state)
    try:
        wb = _load(state)
        ws = wb[sheet_name]
        count = 0

        font_obj = None
        if font:
            font_obj = Font(**font)

        fill_obj = None
        if fill:
            fill_color = fill.get("color", "FFFFFF")
            fill_type = fill.get("fill_type", "solid")
            fill_obj = PatternFill(start_color=fill_color, end_color=fill_color, fill_type=fill_type)

        align_obj = None
        if alignment:
            align_obj = Alignment(**alignment)

        border_obj = None
        if border:
            side = Side(style=border.get("style", "thin"), color=border.get("color", "000000"))
            border_obj = Border(top=side, bottom=side, left=side, right=side)

        for cell in _iter_cells(ws, range_str):
            if font_obj:
                cell.font = font_obj
            if fill_obj:
                cell.fill = fill_obj
            if align_obj:
                cell.alignment = align_obj
            if number_format:
                cell.number_format = number_format
            if border_obj:
                cell.border = border_obj
            count += 1

        wb.save(output)
        applied = []
        if font:
            applied.append("字体")
        if fill:
            applied.append("填充")
        if alignment:
            applied.append("对齐")
        if number_format:
            applied.append("数字格式")
        if border:
            applied.append("边框")
        return f"已为 {count} 个单元格 ({range_str}) 应用: {', '.join(applied)}"
    except KeyError:
        raise ToolException(f"工作表 '{sheet_name}' 不存在")
    except Exception as e:
        raise ToolException(f"format_cells 失败: {e}") from e


@tool
@with_file_lock
def set_column_width(
    state: Annotated[dict, InjectedState],
    sheet_name: str,
    columns: dict[str, float],
) -> str:
    """设置输出文件中指定列的宽度。

    Args:
        sheet_name: 工作表名称
        columns: {列字母: 宽度} 字典，如 {"A": 15.0, "B": 20.0}
    """
    output = get_output_file(state)
    try:
        wb = _load(state)
        ws = wb[sheet_name]
        for col, width in columns.items():
            ws.column_dimensions[col.upper()].width = width
        wb.save(output)
        return f"已设置 {len(columns)} 列宽度: {columns}"
    except KeyError:
        raise ToolException(f"工作表 '{sheet_name}' 不存在")
    except Exception as e:
        raise ToolException(f"set_column_width 失败: {e}") from e


@tool
@with_file_lock
def set_row_height(
    state: Annotated[dict, InjectedState],
    sheet_name: str,
    rows: dict[int, float],
) -> str:
    """设置输出文件中指定行的高度。

    Args:
        sheet_name: 工作表名称
        rows: {行号: 高度} 字典，如 {1: 25.0, 2: 18.0}
    """
    output = get_output_file(state)
    try:
        wb = _load(state)
        ws = wb[sheet_name]
        for row_num, height in rows.items():
            ws.row_dimensions[row_num].height = height
        wb.save(output)
        return f"已设置 {len(rows)} 行高度: {rows}"
    except KeyError:
        raise ToolException(f"工作表 '{sheet_name}' 不存在")
    except Exception as e:
        raise ToolException(f"set_row_height 失败: {e}") from e


@tool
@with_file_lock
def merge_cells(
    state: Annotated[dict, InjectedState],
    sheet_name: str,
    range_str: str,
) -> str:
    """合并输出文件中指定范围的单元格。

    Args:
        sheet_name: 工作表名称
        range_str: 合并范围，如 "A1:D1"
    """
    output = get_output_file(state)
    try:
        wb = _load(state)
        wb[sheet_name].merge_cells(range_str)
        wb.save(output)
        return f"已合并 {range_str}"
    except KeyError:
        raise ToolException(f"工作表 '{sheet_name}' 不存在")
    except Exception as e:
        raise ToolException(f"merge_cells 失败: {e}") from e


@tool
@with_file_lock
def unmerge_cells(
    state: Annotated[dict, InjectedState],
    sheet_name: str,
    range_str: str,
) -> str:
    """取消合并输出文件中指定范围的单元格。

    Args:
        sheet_name: 工作表名称
        range_str: 取消合并范围，如 "A1:D1"
    """
    output = get_output_file(state)
    try:
        wb = _load(state)
        wb[sheet_name].unmerge_cells(range_str)
        wb.save(output)
        return f"已取消合并 {range_str}"
    except KeyError:
        raise ToolException(f"工作表 '{sheet_name}' 不存在")
    except Exception as e:
        raise ToolException(f"unmerge_cells 失败: {e}") from e


@tool
@with_file_lock
def freeze_panes(
    state: Annotated[dict, InjectedState],
    sheet_name: str,
    cell: str,
) -> str:
    """冻结窗格。指定单元格左上方的行和列将被冻结。

    Args:
        sheet_name: 工作表名称
        cell: 冻结位置，如 "B2" 冻结第一行和第一列，"A2" 仅冻结第一行
    """
    output = get_output_file(state)
    try:
        wb = _load(state)
        wb[sheet_name].freeze_panes = cell
        wb.save(output)
        return f"已冻结窗格于 {cell}"
    except KeyError:
        raise ToolException(f"工作表 '{sheet_name}' 不存在")
    except Exception as e:
        raise ToolException(f"freeze_panes 失败: {e}") from e


@tool
@with_file_lock
def auto_fit_columns(
    state: Annotated[dict, InjectedState],
    sheet_name: str,
    columns: Optional[list[str]] = None,
    min_width: float = 8.0,
    max_width: float = 50.0,
) -> str:
    """根据内容自动调整列宽。

    Args:
        sheet_name: 工作表名称
        columns: 指定列字母列表如 ["A","B"]，不指定则调整所有列
        min_width: 最小宽度，默认 8.0
        max_width: 最大宽度，默认 50.0
    """
    from openpyxl.utils import get_column_letter

    output = get_output_file(state)
    try:
        wb = _load(state)
        ws = wb[sheet_name]
        target_cols = set()
        if columns:
            target_cols = {c.upper() for c in columns}

        adjusted = []
        for col_idx in range(1, ws.max_column + 1):
            col_letter = get_column_letter(col_idx)
            if target_cols and col_letter not in target_cols:
                continue
            max_len = 0
            for row_idx in range(1, ws.max_row + 1):
                val = ws.cell(row=row_idx, column=col_idx).value
                if val is not None:
                    max_len = max(max_len, len(str(val)))
            width = min(max(max_len + 2, min_width), max_width)
            ws.column_dimensions[col_letter].width = width
            adjusted.append(f"{col_letter}={width:.1f}")

        wb.save(output)
        return f"已自动调整 {len(adjusted)} 列宽: {', '.join(adjusted)}"
    except KeyError:
        raise ToolException(f"工作表 '{sheet_name}' 不存在")
    except Exception as e:
        raise ToolException(f"auto_fit_columns 失败: {e}") from e



"""场景：完整工作流 — 一次性完成多项操作。"""

from .conftest import invoke


class TestEndToEndWorkflow:

    def test_full_report(self, agent, xlsx_env, renderer):
        """Agent 一次性完成：排序 + 汇总 + 格式化 + 条件高亮。"""
        result, wb = invoke(
            agent,
            "帮我完成以下操作：\n"
            "1. 按销售额从高到低排序\n"
            "2. 在最后添加合计行，求下单次数和销售额的总和\n"
            "3. 表头加粗并加蓝色背景\n"
            "4. 冻结表头行",
            xlsx_env,
            renderer,
        )
        ws = wb["Sheet1"]
        assert ws["A1"].font.bold is True
        assert ws.freeze_panes is not None



import argparse
import os

from pathlib import Path
from ai_hub_agents.test import ColorStreamRenderer, load_test_llm, setup_logging
from ai_hub_agents.xlsx import XlsxAgent

PROJECT_ROOT = Path(__file__).resolve().parent

DEFAULT_INPUT_FILE = PROJECT_ROOT / "example.xlsx"
DEFAULT_OUTPUT_FILE = Path("output") / "example-result.xlsx"
DEFAULT_TASK = "帮我把所有没被退货的商品的行用绿色背景标出，退货最多的用红色标出；计算人数、次数、销售额等的总和"


def main() -> None:
    parser = argparse.ArgumentParser(description="XlsxAgent - Excel 处理 CLI")
    parser.add_argument("-i", "--input_file", default=DEFAULT_INPUT_FILE, help="输入 Excel 文件路径")
    parser.add_argument("-o", "--output_file", default=DEFAULT_OUTPUT_FILE, help="输出 Excel 文件路径")
    parser.add_argument("-t", "--task", default=DEFAULT_TASK, help="任务描述")
    args = parser.parse_args()

    setup_logging()

    input_file = os.path.abspath(args.input_file)
    output_file = os.path.abspath(args.output_file)

    if not os.path.exists(input_file):
        raise FileNotFoundError(f"错误: 输入文件不存在 {input_file}")

    llm = load_test_llm()
    agent = XlsxAgent.create(llm)

    task = args.task

    print(f"输入: {input_file}")
    print(f"输出: {output_file}")
    print(f"任务: {task}")

    state_fields = dict(input_file=input_file, output_file=output_file)

    result = agent.invoke(task, callbacks=[ColorStreamRenderer()], **state_fields)
    print(result)


if __name__ == "__main__":
    main()



"""图表工具。"""

from __future__ import annotations

from typing import Annotated, Optional

from langchain_core.tools import ToolException, tool
from langgraph.prebuilt import InjectedState
from openpyxl import load_workbook

from ._helpers import get_output_file, with_file_lock

_CHART_CLASSES = {
    "bar": "BarChart",
    "column": "BarChart",
    "line": "LineChart",
    "pie": "PieChart",
    "area": "AreaChart",
    "scatter": "ScatterChart",
    "doughnut": "DoughnutChart",
}


@tool
@with_file_lock
def create_chart(
    state: Annotated[dict, InjectedState],
    sheet_name: str,
    chart_type: str,
    data_range: str,
    target_cell: str,
    title: Optional[str] = None,
    x_axis_title: Optional[str] = None,
    y_axis_title: Optional[str] = None,
    width: float = 15.0,
    height: float = 10.0,
    categories_range: Optional[str] = None,
) -> str:
    """在输出文件中创建图表。

    Args:
        sheet_name: 工作表名称
        chart_type: 图表类型，支持 "bar", "column", "line", "pie", "area", "scatter", "doughnut"
        data_range: 数据范围如 "B1:D10"（含标题行）
        target_cell: 图表放置位置，如 "F2"
        title: 图表标题
        x_axis_title: X 轴标题
        y_axis_title: Y 轴标题
        width: 图表宽度（厘米），默认 15.0
        height: 图表高度（厘米），默认 10.0
        categories_range: 类别标签范围如 "A2:A10"
    """
    from openpyxl.chart import Reference
    from openpyxl.utils import range_boundaries

    cls_name = _CHART_CLASSES.get(chart_type)
    if not cls_name:
        raise ToolException(
            f"不支持的图表类型: {chart_type}，可选: {', '.join(_CHART_CLASSES)}"
        )

    import openpyxl.chart as chart_mod

    ChartClass = getattr(chart_mod, cls_name)

    output = get_output_file(state)
    try:
        wb = load_workbook(output)
        ws = wb[sheet_name]

        chart = ChartClass()
        if chart_type == "column":
            chart.type = "col"

        if title:
            chart.title = title
        if x_axis_title and hasattr(chart, "x_axis"):
            chart.x_axis.title = x_axis_title
        if y_axis_title and hasattr(chart, "y_axis"):
            chart.y_axis.title = y_axis_title
        chart.width = width
        chart.height = height

        min_col, min_row, max_col, max_row = range_boundaries(data_range)
        data_ref = Reference(ws, min_col=min_col, min_row=min_row, max_col=max_col, max_row=max_row)
        chart.add_data(data_ref, titles_from_data=True)

        if categories_range:
            c_min_col, c_min_row, c_max_col, c_max_row = range_boundaries(categories_range)
            cats = Reference(ws, min_col=c_min_col, min_row=c_min_row, max_col=c_max_col, max_row=c_max_row)
            chart.set_categories(cats)

        ws.add_chart(chart, target_cell)
        wb.save(output)
        return f"已在 {sheet_name}!{target_cell} 创建 {chart_type} 图表"
    except KeyError:
        raise ToolException(f"工作表 '{sheet_name}' 不存在")
    except ToolException:
        raise
    except Exception as e:
        raise ToolException(f"create_chart 失败: {e}") from e
